// SPDX-FileCopyrightText: 2021 Ray Strode <rstrode@redhat.com>
//
// SPDX-License-Identifier: GPL-2.0-or-later

import Adw from 'gi://Adw';
import Gio from 'gi://Gio';
import GLib from 'gi://GLib';
import GObject from 'gi://GObject';
import Gtk from 'gi://Gtk';

import {ExtensionPreferences, gettext as _} from 'resource:///org/gnome/Shell/Extensions/js/extensions/prefs.js';

Gio._promisify(Gtk.FileDialog.prototype, 'open');

class GeneralGroup extends Adw.PreferencesGroup {
    static {
        GObject.registerClass(this);
    }

    constructor(settings) {
        super();

        const actionGroup = new Gio.SimpleActionGroup();
        this.insert_action_group('options', actionGroup);

        actionGroup.add_action(settings.create_action('show-when-locked'));
        actionGroup.add_action(settings.create_action('show-when-unlocking'));
        actionGroup.add_action(settings.create_action('show-when-unlocked'));

        this.add(new Adw.SwitchRow({
            title: _('Show message when screen is locked'),
            action_name: 'options.show-when-locked',
        }));
        this.add(new Adw.SwitchRow({
            title: _('Show message on unlock screen'),
            action_name: 'options.show-when-unlocking',
        }));
        this.add(new Adw.SwitchRow({
            title: _('Show message when screen is unlocked'),
            action_name: 'options.show-when-unlocked',
        }));

        const spinRow = new Adw.SpinRow({
            title: _('Seconds after user goes idle before reshowing message'),
            adjustment: new Gtk.Adjustment({
                lower: 0,
                upper: 2147483647,
                step_increment: 1,
                page_increment: 60,
                page_size: 60,
            }),
        });
        settings.bind('idle-timeout',
            spinRow, 'value',
            Gio.SettingsBindFlags.DEFAULT);
        this.add(spinRow);
    }
}

class MessageGroup extends Adw.PreferencesGroup {
    static {
        GObject.registerClass(this);
    }

    constructor(settings) {
        super({
            title: _('Message'),
        });

        this._settings = settings;

        const actionGroup = new Gio.SimpleActionGroup();
        const selectAction = new Gio.SimpleAction({name: 'select-file'});
        actionGroup.add_action(selectAction);
        actionGroup.add_action(settings.create_action('message-source'));
        this.insert_action_group('message-group', actionGroup);

        selectAction.connect('activate',
            () => this._selectFile().catch(logError));

        const textCheck = new Gtk.CheckButton({
            action_name: 'message-group.message-source',
            action_target: new GLib.Variant('s', 'settings'),
        });
        const useTextRow = new Adw.ActionRow({
            title: _('Use fixed message text'),
            activatable_widget: textCheck,
        });
        useTextRow.add_prefix(textCheck);

        this.add(useTextRow);

        const textView = new Gtk.TextView({
            accepts_tab: false,
            wrap_mode: Gtk.WrapMode.WORD,
            height_request: 80,
            top_margin: 6,
            bottom_margin: 6,
            left_margin: 6,
            right_margin: 6,
        });

        settings.bind('message-body',
            textView.get_buffer(), 'text',
            Gio.SettingsBindFlags.DEFAULT);

        const textRow = new Adw.PreferencesRow({
            child: textView,
        });
        this.add(textRow);

        textCheck.bind_property('active',
            textRow, 'sensitive',
            GObject.BindingFlags.SYNC_CREATE);

        const fileCheck = new Gtk.CheckButton({
            action_name: 'message-group.message-source',
            action_target: new GLib.Variant('s', 'file'),
            group: textCheck,
        });
        const useFileRow = new Adw.ActionRow({
            title: _('Read message text from file'),
            activatable_widget: fileCheck,
        });
        useFileRow.add_prefix(fileCheck);

        this.add(useFileRow);

        const selectButton = new Gtk.Button({
            label: _('Select…'),
            action_name: 'message-group.select-file',
            valign: Gtk.Align.CENTER,
        });

        const fileRow = new Adw.ActionRow();
        fileRow.add_suffix(selectButton);
        this.add(fileRow);

        fileCheck.bind_property('active',
            fileRow, 'sensitive',
            GObject.BindingFlags.SYNC_CREATE);

        function updateFileLabel() {
            const path = settings.get_string('message-path');
            fileRow.title = path || _('None');
        }

        settings.connect_object('changed::message-path',
            () => updateFileLabel(),
            this, GObject.ConnectFlags.DEFAULT);
        updateFileLabel();
    }

    async _selectFile() {
        const fileDialog = new Gtk.FileDialog({
            modal: true,
            default_filter: new Gtk.FileFilter({
                mime_types: ['text/plain'],
            }),
        });
        const file = await fileDialog.open(this.get_root(), null);
        if (file)
            this._settings.set_string('message-path', file.get_path());
    }
}

export default class HeadsUpDisplayPrefs extends ExtensionPreferences {
    getPreferencesWidget() {
        const page = new Adw.PreferencesPage();
        page.add(new GeneralGroup(this.getSettings()));
        page.add(new MessageGroup(this.getSettings()));
        return page;
    }
}
